@@ -15,17 +15,22 @@ setup(
     long_description_content_type="text/markdown",
     url="https://data-apis.org/array-api-compat/",
     license="MIT",
-    python_requires=">=3.8",
     extras_require={
         "numpy": "numpy",
         "cupy": "cupy",
+        "jax": "jax",
+        "pytorch": "pytorch",
+        "dask": "dask",
+        "sparse": "sparse >=0.15.1",
     },
+    python_requires=">=3.9",
     classifiers=[
         "Programming Language :: Python :: 3",
-        "Programming Language :: Python :: 3.8",
         "Programming Language :: Python :: 3.9",
         "Programming Language :: Python :: 3.10",
         "Programming Language :: Python :: 3.11",
+        "Programming Language :: Python :: 3.12",
+        "Programming Language :: Python :: 3.13",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
     ],
